package sensei.middleware.domains;


/**
* sensei/middleware/domains/PropertiesHandlerOperations.java .
* Error reading Messages File.
* Error reading Messages File.
* Monday, March 15, 2004 12:39:00 AM CET
*/


/***********************************************************************/
public interface PropertiesHandlerOperations 
{

  /**
           * Enables the use of properties, which is disabled by default.
           * This method must be invoked before the member joins the group, or an exception is raised.
           * Every member in the group must use the same properties
           * policy.
           */
  void enableProperties () throws sensei.middleware.domains.MemberStateException;

  /**
           * Returns the properties policy
           */
  boolean arePropertiesEnabled ();

  /**
           * Sets a PropertiesListener, that will receive the properties update notifications. The user
           * can be set at any moment, and if a nil user is specified, no user will be used.
           */
  void setPropertiesListener (sensei.middleware.domains.PropertiesListener listener) throws sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the properties of all the members with state.
           * Properties are only available once the member has received the state. Otherwise, an
           * MemberStateException is thrown
           */
  sensei.middleware.domains.MemberProperties[] getAllProperties () throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the property with the given name of all the members with state.
           * Properties are only available once the member has received the state. Otherwise, an
           * MemberStateException is thrown
           */
  sensei.middleware.domains.MemberProperties[] getPropertyForAllMembers (String nam) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the properties of the specified member.
           * Properties are only available once the member has received the state. Otherwise, an
           * MemberStateException is thrown
           */
  sensei.middleware.domains.Property[] getMemberProperties (int loc) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the property with the given name for the specified member.
           * Properties are only available once the member has received the state. Otherwise, an
           * MemberStateException is thrown
           */
  String getMemberProperty (int loc, String n) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the properties of the member handled by this PropertyHandler.
           * Properties for a given member are available at any moment, not just after the member receives the state
           */
  sensei.middleware.domains.Property[] getProperties () throws sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Returns the property with the given name for the member handled by this PropertyHandler.
           * Properties for a given member are available at any moment, not just after the member receives the state
           */
  String getProperty (String n) throws sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Sets the properties for the member handled by this PropertyHandler.
           * Properties for a given member are available at any moment, not just after the member receives the state.
           * The properties are propagated to the group if the member has state, otherwise the other members will
           * know the changes once the transfer finishes.
           */
  void setProperties (sensei.middleware.domains.Property[] props) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Adds the properties for the member handled by this PropertyHandler. If some property were already
           * existing, its value is overriden.
           * Properties for a given member are available at any moment, not just after the member receives the state.
           * The properties are propagated to the group if the member has state, otherwise the other members will
           * know the changes once the transfer finishes.
           */
  void addProperties (sensei.middleware.domains.Property[] props) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;

  /**
           * Removes the properties for the member handled by this PropertyHandler. If some property were not
           * existing, it is ignored.
           * Properties for a given member are available at any moment, not just after the member receives the state.
           * The properties are propagated to the group if the member has state, otherwise the other members will
           * know the changes once the transfer finishes.
           */
  void removeProperties (sensei.middleware.domains.Property[] props) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.PropertiesDisabledException;
} // interface PropertiesHandlerOperations
