package sensei.middleware.domains;


/**
* sensei/middleware/domains/SubgroupsHandlerOperations.java .
* Error reading Messages File.
* Error reading Messages File.
* Monday, March 15, 2004 12:39:00 AM CET
*/


/**
       *  Interface supporting subgroups handling
       **/
public interface SubgroupsHandlerOperations 
{

  /**
           * Sets a DynamicSubgroupsUser listener. This user must be set before the member joins
           * a group, or an exception is raised
           */
  void setDynamicSubgroupsUser (sensei.middleware.domains.DynamicSubgroupsUser user) throws sensei.middleware.domains.MemberStateException;

  /**
            * Registers a static subgroup with a specific ID. This ID must be unique in the object and also
            * in the group. The group cannot have been activated yet, or an exception is raised.
            * Every member must have defined the same subgroups when the group is joined.
            * Once joined, it is not possible anymore to use registerSubgroup, as the subgroups
            * must be dynamically registered.
            * The subgroups registered with this method cannot be later unregistered.
            * The subgroupId must be a valid static subgroup Id (lower or equal to MAX_STATIC_SUBGROUP_ID)
            * or the subgroup is not registered and an exception is raised
            * @exception MemberStateException if the member has not yet joined a group, or it has been expulsed
            * @exception SubgroupsHandlerException if the subgroupId specified is not valid (not static)
            **/
  void registerSubgroup (int uniqueSubgroupId, sensei.middleware.gms.GroupMember subgroup) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.SubgroupsHandlerException;

  /**
            * Registers a group dynamically, only possible when the member belongs already to a group and
            * has registered a dynamic subgroups user.
            * When this method is invoked, every member in the group, included this one, receives a invocation
            * through createdSubgroup, and a Subgroup must be supplied.
            * @exception MemberStateException if the member has not yet joined a group, or it has been expulsed
            * @exception SubgroupsHandlerException if the member does not support dynamic groups
            **/
  void castSubgroupCreation (sensei.middleware.domains.DynamicSubgroupInfo info) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.SubgroupsHandlerException;

  /**
            * Registers a group dynamically, only possible when the member belongs already to a group and
            * has registered a dynamic subgroups user.
            * When this method is invoked, every member in the group, included this one, receives a invocation
            * through createdSubgroup, and a Subgroup must be supplied.
            * this call blocks until the subgroup is registered, returning the created subgroupId
            * @exception MemberStateException if the member has not yet joined a group, or it has been expulsed
            * @exception SubgroupsHandlerException if the member does not support dynamic groups
            **/
  int createSubgroup (sensei.middleware.domains.DynamicSubgroupInfo info, sensei.middleware.gms.GroupMember subgroup) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.SubgroupsHandlerException;

  /**
            * Removes the given subgroup. Only dynamic subgroups can be removed.
            * If the group is static, an exception is raised. If the subgroup does not exist, false is returned
            * Every member in the group receives a removedSubgroup notification
            * @exception MemberStateException if the member has not yet joined a group, or it has been expulsed
            * @exception SubgroupsHandlerException if the member does not support dynamic groups or the
            *       subgroupId given is describes a static subgroup
            */
  boolean removeSubgroup (int id, sensei.middleware.domains.DynamicSubgroupInfo info) throws sensei.middleware.domains.MemberStateException, sensei.middleware.domains.SubgroupsHandlerException;

  /**
            * Returns the existing groups, statics and dynamics
            */
  int[] getSubgroups ();

  /**
            * Returns the subgroup associated to a given subgroupId, null if not existing
            */
  sensei.middleware.gms.GroupMember getSubgroup (int id);

  /**
            * Returns the subgroupId for a group. If the group is not valid, EVERY_SUBGROUP is returned
            */
  int getSubgroupId (sensei.middleware.gms.GroupMember subgroup);
} // interface SubgroupsHandlerOperations
