/**
  * File: BasicStateHandlerBaseImpl.java
  * Content: Defines an abstract corba class to implement the BasicStateHandler interface
  * Author: LuisM Pena
  * Date: 25th July 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.middleware.domains;

import org.omg.CORBA.UserException;

import sensei.middleware.gms.ORBcentral;

/**
  * Implementation of the BasicStateHandler CORBA interface.
  **/
public abstract class BasicStateHandlerBaseImpl extends BasicStateHandlerPOA {

	public BasicStateHandlerBaseImpl() throws UserException {
		this(true);
	}

	public BasicStateHandlerBaseImpl(boolean active) throws UserException {
		if (active)
			activate();
	}

	public void activate() throws UserException {
		if (thisBasicStateHandler == null) {
			id = ORBcentral.getPOA().activate_object(this);
			thisBasicStateHandler = BasicStateHandlerHelper.narrow(ORBcentral.getPOA().id_to_reference(id));
		}
	}

	public void deactivate() throws UserException {
		synchronized (this) {
			if (thisBasicStateHandler != null)
				ORBcentral.getPOA().deactivate_object(id);
			thisBasicStateHandler = null;
		}
	}

	public final BasicStateHandler theBasicStateHandler() {
		return thisBasicStateHandler;
	}

	protected BasicStateHandler thisBasicStateHandler;
	byte[] id;
};
