/**
  * File: CheckpointableBaseImpl.java
  * Content: Defines an abstract corba class to implement the Checkpointable interface
  * Author: LuisM Pena
  * Date: 25th July 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.middleware.domains;

import org.omg.CORBA.UserException;

import sensei.middleware.gms.ORBcentral;

/**
  * Implementation of the Checkpointable CORBA interface.
  **/
public abstract class CheckpointableBaseImpl extends CheckpointablePOA {

	public CheckpointableBaseImpl() throws UserException {
		this(true);
	}

	public CheckpointableBaseImpl(boolean active) throws UserException {
		if (active)
			activate();
	}

	public void activate() throws UserException {
		if (thisCheckpointable == null) {
			id = ORBcentral.getPOA().activate_object(this);
			thisCheckpointable = CheckpointableHelper.narrow(ORBcentral.getPOA().id_to_reference(id));
		}
	}

	public void deactivate() throws UserException {
		synchronized (this) {
			if (thisCheckpointable != null)
				ORBcentral.getPOA().deactivate_object(id);
			thisCheckpointable = null;
		}
	}

	public final Checkpointable theCheckpointable() {
		return thisCheckpointable;
	}

	protected Checkpointable thisCheckpointable;
	byte[] id;
};
