/**
  * File: GroupMonitorBaseImpl.java
  * Content: Monitor implementation on a replicated environment
  * Author: LuisM Pena
  * Date: 03rd October 2000
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.middleware.domains;

import org.omg.CORBA.UserException;

import sensei.middleware.gms.ORBcentral;


/**
  * Monitor implementation on a replicated environment
  **/
public abstract class GroupMonitorBaseImpl extends GroupMonitorPOA {

	//*************************************************************************************//
	//**************************** CONSTRUCTOR ********************************************//
	//*************************************************************************************//

	public GroupMonitorBaseImpl() throws UserException {
		this(true);
	}

	public GroupMonitorBaseImpl(boolean active) throws UserException {
		if (active)
			activate();
	}

	public void activate() throws UserException {
		if (thisGroupMonitor == null) {
			id = ORBcentral.getPOA().activate_object(this);
			thisGroupMonitor = GroupMonitorHelper.narrow(ORBcentral.getPOA().id_to_reference(id));
		}
	}

	public void deactivate() throws UserException {
		synchronized (this) {
			if (id != null)
				ORBcentral.getPOA().deactivate_object(id);
			id = null;
		}
	}

	public final GroupMonitor theGroupMonitor() {
		return thisGroupMonitor;
	}

	//*************************************************************************************//
	//**************************** DATA MEMBERS *******************************************//
	//*************************************************************************************//

	protected GroupMonitor thisGroupMonitor;
	byte[] id;
}
