/**
  * File: StateHandlerBaseImpl.java
  * Content: Defines an abstract corba class to implement the StateHandler interface
  * Author: LuisM Pena
  * Date: 25th July 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.middleware.domains;

import org.omg.CORBA.UserException;

import sensei.middleware.gms.ORBcentral;

/**
  * Implementation of the StateHandler CORBA interface.
  **/
public abstract class StateHandlerBaseImpl extends StateHandlerPOA {

	public StateHandlerBaseImpl() throws UserException {
		this(true);
	}

	public StateHandlerBaseImpl(boolean active) throws UserException {
		if (active)
			activate();
	}

	public void activate() throws UserException {
		if (thisStateHandler == null) {
			id = ORBcentral.getPOA().activate_object(this);
			thisStateHandler = StateHandlerHelper.narrow(ORBcentral.getPOA().id_to_reference(id));
		}
	}

	public void deactivate() throws UserException {
		synchronized (this) {
			if (thisStateHandler != null)
				ORBcentral.getPOA().deactivate_object(id);
			thisStateHandler = null;
		}
	}

	public final StateHandler theStateHandler() {
		return thisStateHandler;
	}

	protected StateHandler thisStateHandler;
	byte[] id;
};
