/**
  * File: GMNSRegister.java
  * Content: Class to register the valuetypes on the ORB
  * Author: LuisM Pena
  * Date: 14th December 2000
  * Version: 0.29.00
  * Last change:
  *
  **/

package sensei.middleware.gmns;

import java.util.Properties;

import org.omg.CORBA.portable.ValueFactory;
import org.omg.CORBA_2_3.ORB;
import org.omg.CORBA_2_3.portable.InputStream;

import sensei.middleware.gms.ORBcentral;

/**
  * Class to register the valuetypes on the ORB
  **/
public abstract class GMNSRegisterBase {
	//*************************************************************************************
	//**************************** ORB REGISTERING ****************************************
	//*************************************************************************************

	public GMNSRegisterBase(GMNSMain mainClass, String args[], boolean graphicalMode) {
		Properties props = System.getProperties();
		org.omg.CORBA.ORB  orb = org.omg.CORBA.ORB.init(args, props);

		ORBcentral.setORB(orb);
		register();
		mainClass.runGMNS(graphicalMode);
		orb.run();

		if (orb != null){
			orb.destroy();
		}
	}

	protected abstract GroupHandlerFactoryCreator createGroupHandlerFactoryCreator();

	void register() {
		ORB orb = ((ORB) (ORBcentral.getORB()));
		orb.register_value_factory(GroupHandlerFactoryCreatorHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(createGroupHandlerFactoryCreator());
			}
		});
		orb.register_value_factory(MapStringSetPutMessageHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new MapStringSetPutMessage() {
				});
			}
		});
		orb.register_value_factory(MapStringSetRemoveMessageHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new MapStringSetRemoveMessage() {
				});
			}
		});
		orb.register_value_factory(MapStringSetStateHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new MapStringSetState() {
				});
			}
		});
		orb.register_value_factory(SetMemberInfoChangedMessageHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new SetMemberInfoChangedMessage() {
				});
			}
		});
		orb.register_value_factory(SetMemberInfoStateHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new SetMemberInfoState() {
				});
			}
		});
		orb.register_value_factory(GroupsCheckerStateHelper.id(), new ValueFactory() {
			public java.io.Serializable read_value(InputStream in) {
				return in.read_value(new GroupsCheckerState() {
				});
			}
		});
	}
}
