package sensei.middleware.gms;


/**
* sensei/middleware/gms/SenseiGMSMemberOperations.java .
* Error reading Messages File.
* Error reading Messages File.
* Sunday, March 7, 2004 3:19:30 PM CET
*/


/**
        * GMSMember specifies the interface that group members must
        * implement to enable the communications in the group.
        * The same member is a GroupHandler itself
        **/
public interface SenseiGMSMemberOperations  extends sensei.middleware.gms.GroupHandlerOperations
{

  /**
            * The member receives the token. It is time to send any
            * queued messages or to install new views, and, after that,
            * transfer the token to the next member in the group.
            * @param sender The member sending the token
            * @param token  The token being passed
            * @returns true if the token is accepted.
            **/
  boolean receiveToken (int sender, sensei.middleware.gms.Token theToken);

  /**
            * The member receives a new view. If the member is joining
            * the group, this view includes effectively the member in
            * the group, and its call to join will eventually return
            * with a succesfull flag
            * @param sender The member sending the token
            * @param view The new view to install
            * @param viewToken The token to be used by the group during the
            *        new view. If the new view has been generated after a
            *        recovery of a lost token, this token must be differente
            *        of the one used in the previous view
            * @returns true if the view is accepted
            **/
  boolean receiveView (int sender, sensei.middleware.gms.InternalView view, sensei.middleware.gms.Token viewToken);

  /**
            * The member receives new messages. They won't be processed immediately
            * (it's a two phase commit process)
            * @param sender The member sending the token
            * @param messages The array of messages to process
            * @param messageId an id for the group of messages
            * @returns true if the messages are accepted
            **/
  boolean receiveMessages (int sender, sensei.middleware.gms.Message[] messages, sensei.middleware.gms.MessageId msgId);

  /**
            * The member receives the confirmation to previous messages.
            * @param sender The member sending the token
            * @param messageId an id for the group of messages
            * @returns true if the confirmation is valid
            **/
  boolean confirmMessages (int sender, sensei.middleware.gms.MessageId msgId);

  /**
            * The member receives one or more point to point messages.
            * @param sender The member sending the token
            * @param messages The point to point messages to process
            * @param messageId It's only required the view information in the message id.
            *    * 	The id itself must be zero.
            * @return true if the message is accepted
            **/
  boolean receivePTPMessages (int sender, sensei.middleware.gms.Message[] messages, sensei.middleware.gms.MessageId msgId);

  /**
            * This method is called by another member which thinks that
            * the token is lost and it's trying to recover it. There are
            * three possible answers, depending on the returning values and
            * the recover parameter.
            * @param sender The member trying to recover the token
            * @param myViewId the view id currently seen by the sender
            * @param recover includes the answer of this member to the
            *    try to recover the token
            * @returns true if it considers that the sender can recover the token
            **/
  sensei.middleware.gms.AllowTokenRecoveryAnswer allowTokenRecovery (int sender, sensei.middleware.gms.InternalViewId myViewId);

  /**
            * This method is called by another member which is recovering
            * the token; after this message is received, the token is considered
            * lost and no tokens or views will be accepted until a new view
            * is installed by the member who has sent this message.
            * @param sender The member trying to recover the token
            * @returns false if it doesn't accept the token recovery or the sender
            **/
  boolean recoverToken (int sender);

  /**
            * Joins the member to the current group where this member
            * is included. It blocks until the member is effectively
            * included in the group (it will also receive a view with
            * the members in the group).
            * @param other The member willing to join the group
            * @returns false if the member can not be included because
            *  the contacted GroupMember doesn't belong to any group
            */
  boolean addGroupMember (sensei.middleware.gms.SenseiGMSMember other);

  /**
            * This method starts a new group where the only member is this member. This member
            * cannot be part of any other group, or the creation will fail
            **/
  boolean createGroup ();

  /**
            * This method joins an existing group. This joining can fail, and in that case
            * it is returned false and this object is invalidated, that is, cannot be used
            * to join another member. It can fail as well if the member belonged already to
            * other group; in that case, the member is not invalidated (it can be checked
            * previously using validObject()).
            * This method will use addGroupMember on the specified goup, taking care of
            * the possible exclusion conditions.
            **/
  boolean joinGroup (sensei.middleware.gms.SenseiGMSMember group);
} // interface SenseiGMSMemberOperations
