/**
  * File: 		DomainMessage.java
  * Content: 	part of the RMI specification of sensei domains
  *           A DomainMessage includes some additional information over the plain Message to handle
  *           the extended domain functionality
  * Author: 	LuisM Pena
  * Date: 		27th October 2001
  * Version:  0.51.00
  * Last change:
  *
  **/


package sensei.middleware.domains;

import sensei.middleware.gms.Message;

/**
 * A DomainMessage includes some additional information over the plain Message to handle
 * the extended domain functionality
 */
public class DomainMessage extends Message
{
  //subgroup is the only field to be usually filled by the application
  public int subgroup;		      //subgroup id. EVERY_SUBGROUP by default

  //following fields should be false by default
  public boolean unqueuedOnST;//to true if the message is NOT queued during state transfers
  public boolean unTransactionable;//to true if the message is NOT queued during transactions. It is only read
                                   //if unqueuedOnST is also true

  //following fields are used by the domains package
  public boolean waitReception; //to false (default) if the sending does not wait until the member receives it.
  public int id;					      //sequential id, used by the DomainGroupHandler
}
