/**
  * File: STGroupMemberStateHandler.java
  * Content: StateTransferGroupMember with StateHandler behaviour
  * Author: LuisM Pena
  * Date: 31st Octonber 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import java.io.IOException;

import sensei.middleware.gms.GroupHandler;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;
import sensei.middleware.domains.PhaseCoordination;
import sensei.middleware.domains.PhaseCoordinationHolder;
import sensei.middleware.domains.State;
import sensei.middleware.domains.StateHandler;
import sensei.middleware.domains.StateTransferType;

/**
  * StateTransferGroupMember without state transfer
  **/
class STGroupMemberStateHandler extends STGroupMember {

	//*************************************************************************************//
	//**************************** CONSTRUCTOR ********************************************//
	//*************************************************************************************//

	public STGroupMemberStateHandler(StateHandler groupMember) {
		Trace.code("STGroupMemberStateHandler.java -> STGroupMemberStateHandler ( StateHandler groupMember )");
		this.groupMember = groupMember;
	}

	public GroupMember getGroupMember() {
		return groupMember;
	}
	public boolean needsTransfer() {
		return true;
	}
	public StateTransferType getStateTransferType() {
		return StateTransferType.STT_StateHandler;
	}
	public void freeResources() {
		groupMember = null;
	}

	//*************************************************************************************//
	//**************************** STATE TRANSFER METHODS *********************************//
	//*************************************************************************************//

	public void processPTPMessage(int sender, Message msg) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void processPTPMessage ( int sender , Message msg )");
		groupMember.processPTPMessage(sender, msg);
	}
	public void processCastMessage(int sender, Message msg) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void processCastMessage ( int sender , Message msg )");
		groupMember.processCastMessage(sender, msg);
	}
	public void memberAccepted(int identity, GroupHandler handler, View theView) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void memberAccepted ( int identity , GroupHandler handler , View theView )");
		groupMember.memberAccepted(identity, handler, theView);
	}
	public void changingView() throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void changingView ( )");
		groupMember.changingView();
	}
	public void installView(View theView) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void installView ( View theView )");
		groupMember.installView(theView);
	}
	public void excludedFromGroup() throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void excludedFromGroup ( )");
		groupMember.excludedFromGroup();
	}

	//*************************************************************************************//
	//**************************** STATE TRANSFER METHODS *********************************//
	//*************************************************************************************//

	public void assumeState() throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void assumeState ( )");
		groupMember.assumeState();
	}
	public void startTransfer(int[] joiningMembers, PhaseCoordinationHolder phase) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void startTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder phase )");
		groupMember.startTransfer(joiningMembers, phase);
	}
	public State getState(PhaseCoordinationHolder phase) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> State getState ( PhaseCoordinationHolder phase )");
		return groupMember.getState(phase);
	}
	public void setState(State s, PhaseCoordination phase) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void setState ( State s , PhaseCoordination phase )");
		groupMember.setState(s, phase);
	}
	public void stopTransfer(int[] joiningMembers, boolean transferFinished) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void stopTransfer ( int [ ] joiningMembers , boolean transferFinished )");
		groupMember.stopTransfer(joiningMembers, transferFinished);
	}
	public void syncTransfer(int coordinator, PhaseCoordinationHolder phase) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void syncTransfer ( int coordinator , PhaseCoordinationHolder phase )");
		groupMember.syncTransfer(coordinator, phase);
	}
	public void interruptTransfer(PhaseCoordinationHolder phase) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void interruptTransfer ( PhaseCoordinationHolder phase )");
		groupMember.interruptTransfer(phase);
	}
	public void continueTransfer(int[] joiningMembers, PhaseCoordinationHolder coordinator, PhaseCoordination joining) throws IOException {
		Trace.code("STGroupMemberStateHandler.java -> void continueTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder coordinator , PhaseCoordination joining )");
		groupMember.continueTransfer(joiningMembers, coordinator, joining);
	}

	//*************************************************************************************//
	//**************************** DATA MEMBERS *******************************************//
	//*************************************************************************************//

	StateHandler groupMember;
}
