/**
  * File: DataListener.java
  * Content: Transforms the events sent by the Map and the Set into calls to a GMNSObserver interface
  * Author: LuisM Pena
  * Date: 20th November 2001
  * Version: 0.29.00
  * Last change:
  *
  **/

package sensei.gmns;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import sensei.middleware.gmns.MapStringSetObservable;
import sensei.middleware.gmns.MapStringSetObserverBaseImpl;
import sensei.middleware.gmns.MemberInfo;
import sensei.middleware.gmns.SetMemberInfo;
import sensei.middleware.gmns.SetMemberInfoObservable;
import sensei.middleware.gmns.SetMemberInfoObserverBaseImpl;
import sensei.util.SenseiError;

/**
  * Transforms the events sent by the Map and the Set into calls to a GMNSObserver interface
  **/
public class DataListener extends MapStringSetObserverBaseImpl {
	/***********************************************************************************************/
	/********************** CONSTRUCTOR ************************************************************/
	/***********************************************************************************************/

	public DataListener(GMNSObserver observer) throws Exception {
		Trace.code("DataListener.java -> DataListener ( GMNSObserver observer )");
		this.observer = observer;
		groups = new HashMap();
	}

	public void clearDone(MapStringSetObservable observable) {
		Trace.code("DataListener.java -> void clearDone ( MapStringSetObservable observable )");
		try {
			Iterator it = groups.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry entry = (Map.Entry) it.next();
				((GroupListener) entry.getValue()).deactivate();
				observer.removeGroup((String) entry.getKey());
			}
			groups.clear();
		}
		catch (Exception ex) {
			SenseiError.unhandledException(Consts.AREA, ex);
		}
	}

	public void putDone(MapStringSetObservable observable, String groupName, SetMemberInfo set) {
		Trace.code("DataListener.java -> void putDone ( MapStringSetObservable observable , String groupName , SetMemberInfo set )");
		try {
			observer.addGroup(groupName);
			groups.put(groupName, new GroupListener(groupName, set));
		}
		catch (Exception ex) {
			SenseiError.unhandledException(Consts.AREA, ex);
		}
	}

	public void removeDone(MapStringSetObservable observable, String groupName, SetMemberInfo set) {
		Trace.code("DataListener.java -> void removeDone ( MapStringSetObservable observable , String groupName , SetMemberInfo set )");
		try {
			observer.removeGroup(groupName);
			GroupListener listener = (GroupListener) groups.remove(groupName);
			if (listener != null)
				listener.deactivate();
		}
		catch (Exception ex) {
			SenseiError.unhandledException(Consts.AREA, ex);
		}
	}

	//*************************************************************************************//
	//**************************** INNER CLASS ********************************************//
	//*************************************************************************************//

	class GroupListener extends SetMemberInfoObserverBaseImpl {
		public GroupListener(String group, SetMemberInfo set) throws Exception {
			this.group = group;
			set.addObserver(thisSetMemberInfoObserver);
		}

		public void addDone(SetMemberInfoObservable observable, MemberInfo member) {
			observer.addGroupMember(group, member.name, member.publicReference != null);
		}

		public void removeDone(SetMemberInfoObservable observable, MemberInfo member) {
			observer.removedGroupMember(group, member.name);
		}
		String group;
	}

	//*************************************************************************************//
	//**************************** DATA MEMBERS *******************************************//
	//*************************************************************************************//

	GMNSObserver observer;
	Map groups; //string, GroupObserver
}
