/**
  * File: SetMemberInfoObserversHandler.java
  * Content: Helper class to handle Observers on the SetMemberInfo class
  *
  * Author: LuisM Pena
  * Date: 20th November 2001
  * Version: 0.29.00
  * Last change:
  *
  *
  **/

package sensei.gmns;

import sensei.util.SenseiError;

import sensei.middleware.gmns.MemberInfo;
import sensei.middleware.gmns.SetMemberInfoObservable;
import sensei.middleware.gmns.SetMemberInfoObserver;
import sensei.middleware.gms.ObjectsHandling;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

class SetMemberInfoObserversHandler {
	public SetMemberInfoObserversHandler(SetMemberInfoObservable observable) {
		Trace.code("SetMemberInfoObserversHandler.java -> SetMemberInfoObserversHandler ( SetMemberInfoObservable observable )");
		observers = new ArrayList();
		this.observable = observable;
	}

	public synchronized void addObserver(SetMemberInfoObserver observer, MemberInfo[] current) {
		Trace.code("SetMemberInfoObserversHandler.java -> synchronized void addObserver ( SetMemberInfoObserver observer , MemberInfo [ ] current )");
		observers.add(observer);
		try {
			for (int i = 0; i < current.length; i++)
				observer.addDone(observable, current[i]);
		}
		catch (Exception ex) {
			SenseiError.unhandledException(Consts.AREA, ex);
		}
	}

	public synchronized boolean removeObserver(SetMemberInfoObserver observer) {
		Trace.code("SetMemberInfoObserversHandler.java -> synchronized boolean removeObserver ( SetMemberInfoObserver observer )");
		Iterator it = observers.iterator();
		while (it.hasNext())
			if (ObjectsHandling.areEquivalent(observer, (SetMemberInfoObserver) it.next())) {
				it.remove();
				return true;
			}
		return false;
	}

	public synchronized void informPut(MemberInfo info) {
		Trace.code("SetMemberInfoObserversHandler.java -> synchronized void informPut ( MemberInfo info )");
		Iterator it = observers.iterator();
		while (it.hasNext())
			try {
				((SetMemberInfoObserver) it.next()).addDone(observable, info);
			}
			catch (Exception ex) {
				//SenseiError.unhandledException(Consts.AREA, ex);
			}
	}

	public synchronized void informRemove(MemberInfo info) {
		Trace.code("SetMemberInfoObserversHandler.java -> synchronized void informRemove ( MemberInfo info )");
		Iterator it = observers.iterator();
		while (it.hasNext())
			try {
				((SetMemberInfoObserver) it.next()).removeDone(observable, info);
			}
			catch (Exception ex) {
				//SenseiError.unhandledException(Consts.AREA, ex);
			}
	}

	List observers;
	SetMemberInfoObservable observable;
};
