/**
  * File: ChannelRequester.java
  * Content: Defines intermediate objects to make requests to the individual
  *		CommunicationChannels from the CommunicationChannelsManager
  * Author: LuisM Pena
  * Date: 20nd July 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import java.io.IOException;

import sensei.middleware.gms.SenseiGMSMember;


//*************************************************************************************//
//**************************** CHANNEL REQUESTER **************************************//
//*************************************************************************************//

/**
  * Basic interface for the requesters. Every request is split in two
  * phases: make request / wait for the result
  **/
abstract class ChannelRequester {
	/**
	  * The channel, after receive the request, will invoke asynchronously the member
	  * through this method
	  * @param channelMember the channel with the link to the member
	  * @return true if the invokation is successfull
	  **/
	public boolean invokeMember(CommunicationChannel channel){
		Trace.code("ChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
		assert channel != null;

		boolean ret;
		SenseiGMSMember channelMember = channel.getLink();

		assert channelMember != null;

		try {
			ret = invokeMember(channelMember) && channel.isReliable();
		}
		catch (Exception ex) { //RemoteException or CorbaException
			ret = false;
			Trace.handledException(ex);
		}
		return ret;
	}
	
	protected abstract boolean invokeMember(SenseiGMSMember member) throws IOException;
}
