/**
  * File: TokenRecovererCommunicationChannelsManager.java
  * Content: CommunicationChannelsManager handling requests to recover token.
  *          It's an specialization for the Token Recoverer
  * Author: LuisM Pena
  * Date: 19th August 2000
  * Version: 0.97.00
  * Last change:
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.InternalViewId;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;

/**
  * CommunicationChannelsManager handling requests to recover token.
  * It's an specialization for the Token Recoverer
  **/
class TokenRecovererCommunicationChannelsManager extends CommunicationChannelsManager {
	/**
	  * The manager is built defining from first moment the channels
	  * to be included in the ActiveGroupMember
	  * @param owner the SenseiGMSMember owning this channel
	  * @param members The members to which is needed to create channels
	  **/
	public TokenRecovererCommunicationChannelsManager(SenseiGMSMember thisSenseiGMSMember, InternalView initialView) {
		super(thisSenseiGMSMember, initialView);
		Trace.code("TokenRecovererCommunicationChannelsManager.java -> TokenRecovererCommunicationChannelsManager ( SenseiGMSMember thisSenseiGMSMember , InternalView initialView )");
	}

	//*************************************************************************************//
	//**************************** SEND REQUEST TO REGENERATE TOKEN ***********************//
	//*************************************************************************************//

	/**
	  * Sends the request to regenerate the token to every member.
	  * Once the request is sent, it can be accesed the method
	  * processLastMessageAfterTokenRegeneration to check if the last message
	  * must be processed or not
	  * @param viewId the view seen by the owner
	  * @param messageId last message Id processed by the owner. It's an out parameter
	  * @param processLastMessage on return, this parameter will be set to true if the
		  *      	last message in the group must be processed. It has no sense if the return value
		  *      	is not true;
	  * @returns true if no contacted member has replied 'NO'
	  **/
	public boolean sendRequestToGenerateToken(InternalViewId viewId, MessageId messageId, OutBoolean processLastMessage) {
		Trace.code("TokenRecovererCommunicationChannelsManager.java -> boolean sendRequestToGenerateToken ( InternalViewId viewId , MessageId messageId , OutBoolean processLastMessage )");
		//as difference with the other requester, this is always created.
		//First, because it modifies one of the parameters, and second, because although
		//it would be easy to override that problem, this recovery can be considered an
		//expensive process. Additionally, it is never called twice!
		AllowTokenRecoveryChannelRequester allowTokenRecoveryChannelRequester = new AllowTokenRecoveryChannelRequester(thisMember, viewId, messageId);

		makeRequestToEveryChannel(allowTokenRecoveryChannelRequester);

		return allowTokenRecoveryChannelRequester.getAnswer(processLastMessage);
	}

}
