/**
  * File: TraceGraphical.java
  * Content: Class to log the activity of a SenseiGMSMember in a graphical way
  * Author: LuisM Pena
  * Date: 11nd July 2000
  * Version: 0.97.00
  * Last change:
  *
  * Comments: This class is not intended to be included in the release
  *						version.
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.MessageId;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.JCheckBox;
import javax.swing.SwingConstants;

/**
  * TraceGMSventsGraphical defines an TraceGMSvents class to log the activity
  * of GroupMembers in a graphical way
  **/
public class TraceGraphical extends JPanel implements ItemListener {

	final boolean printMessages;
	final boolean showExclusion;

	JLabel viewMembers, rank, viewId, messageId;
	JProgressBar tokensReceived;
	JRadioButton tokenOwned, messageConfirmed;
	JCheckBox slowToken, updateToken, catchToken;
	boolean slow, update, catchIt;

	final static int MAX_PROGRESS = 1000;

	public TraceGraphical(boolean printMessages, boolean showExclusion) {
		Trace.code("TraceGraphical.java -> TraceGraphical ( boolean printMessages , boolean showExclusion )");
		this.printMessages = printMessages;
		this.showExclusion = showExclusion;
		if (sensei.util.Configuration.getSingleton().traceGMS())
			tracePanel(sensei.util.Configuration.getSingleton().traceGMSToken());
		else
			noTracePanel();
		setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Group Member Info"));
	}

	void noTracePanel() {
		Trace.code("TraceGraphical.java -> void noTracePanel ( )");
		Box mainBox = Box.createVerticalBox();
		mainBox.add(new JLabel("Trace not activated, no group info available"));
		add(mainBox);
	}

	void tracePanel(boolean traceToken) {
		Trace.code("TraceGraphical.java -> void tracePanel ( boolean traceToken )");
		Box mainBox = Box.createVerticalBox();
		Box firstRow = Box.createHorizontalBox();
		Box secondRow = Box.createHorizontalBox();
		Box thirdRow = Box.createHorizontalBox();

		Dimension separation = new Dimension(10, 10);

		viewMembers = new JLabel("??", SwingConstants.CENTER);
		viewMembers.setFont(new Font("Serif", Font.BOLD, 18));
		viewMembers.setBorder(BorderFactory.createLoweredBevelBorder());
		firstRow.add(new JLabel("InternalView members: "));
		firstRow.add(viewMembers);
		firstRow.add(Box.createRigidArea(separation));

		rank = new JLabel("??", SwingConstants.CENTER);
		rank.setFont(new Font("Serif", Font.BOLD, 18));
		rank.setBorder(BorderFactory.createLoweredBevelBorder());
		firstRow.add(new JLabel("Rank: "));
		firstRow.add(rank);
		firstRow.add(Box.createRigidArea(separation));

		viewId = new JLabel("?.?", SwingConstants.CENTER);
		viewId.setFont(new Font("Serif", Font.BOLD, 18));
		viewId.setBorder(BorderFactory.createLoweredBevelBorder());
		firstRow.add(new JLabel("InternalView Id: "));
		firstRow.add(viewId);

		messageId = new JLabel("?.?", SwingConstants.CENTER);
		messageId.setFont(new Font("Serif", Font.BOLD, 18));
		messageId.setBorder(BorderFactory.createLoweredBevelBorder());
		secondRow.add(new JLabel("Last message Id: "));
		secondRow.add(messageId);
		secondRow.add(Box.createRigidArea(separation));

		messageConfirmed = new JRadioButton("Confirmed");
		messageConfirmed.setEnabled(false);
		secondRow.add(messageConfirmed);
		secondRow.add(Box.createRigidArea(separation));

		mainBox.add(firstRow);
		mainBox.add(Box.createRigidArea(separation));
		mainBox.add(secondRow);

		if (traceToken) {
			tokenOwned = new JRadioButton("Token!");
			tokenOwned.setEnabled(false);
			thirdRow.add(tokenOwned);
			thirdRow.add(Box.createRigidArea(separation));

			tokensReceived = new JProgressBar(SwingConstants.HORIZONTAL, 0, MAX_PROGRESS);
			thirdRow.add(tokensReceived);
			thirdRow.add(Box.createRigidArea(separation));

			slow = false;
			slowToken = new JCheckBox("Slow", slow);
			slowToken.addItemListener(this);
			thirdRow.add(slowToken);
			thirdRow.add(Box.createRigidArea(separation));

			update = false;
			updateToken = new JCheckBox("Update", update);
			updateToken.addItemListener(this);
			thirdRow.add(updateToken);
			thirdRow.add(Box.createRigidArea(separation));

			catchToken = new JCheckBox("Catch it", catchIt);
			catchToken.addItemListener(this);
			thirdRow.add(catchToken);

			mainBox.add(Box.createRigidArea(separation));
			mainBox.add(thirdRow);
		}

		add(mainBox);
	}

	public void itemStateChanged(ItemEvent e) {
		Trace.code("TraceGraphical.java -> void itemStateChanged ( ItemEvent e )");
		if (e.getItem().equals(slowToken)) {
			slow = !slow;
			if (!slow)
				tokenOwned.setSelected(false);
		}
		else if (e.getItem().equals(updateToken)) {
			update = !update;
		}
		else if (e.getItem().equals(catchToken)) {
			catchIt = !catchIt;
			synchronized (this) {
				notify();
			}
		}
	}

	public void memberTooSlow() {
		Trace.code("TraceGraphical.java -> void memberTooSlow ( )");
		printAlways("Member too slow");
	}

	public void memberExitGroup() {
		Trace.code("TraceGraphical.java -> void memberExitGroup ( )");
		if (showExclusion)
			printAlways("Member excluded from the group");
	}

	public void token(boolean received) {
		Trace.code("TraceGraphical.java -> void token ( boolean received )");
		tokenOwned.setSelected(received);
		if (received) {
			if (update) {
				int nextValue = tokensReceived.getValue() + 1;
				if (nextValue > MAX_PROGRESS)
					nextValue = 0;
				tokensReceived.setValue(nextValue);
			}
			if (slow) {
				try {
					Thread.sleep(1000);
				}
				catch (InterruptedException ex) {
				}
			}
			if (catchIt) {
				synchronized (this) {
					try {
						wait();
					}
					catch (InterruptedException ex) {
					}
				}
			}
		}
	}

	public void viewAccepted(InternalView view, int rank) {
		Trace.code("TraceGraphical.java -> void viewAccepted ( InternalView view , int rank )");
		viewInstalled(view, rank);
	}

	public void viewInstalled(InternalView view, int rank) {
		Trace.code("TraceGraphical.java -> void viewInstalled ( InternalView view , int rank )");
		viewMembers.setText(Integer.toString(view.members.length));
		this.rank.setText(Integer.toString(rank));
		this.viewId.setText(Integer.toString(view.viewId.id) + "." + Integer.toString(view.viewId.installing));
	}

	public void messageReceived(MessageId messageId, boolean confirmation) {
		Trace.code("TraceGraphical.java -> void messageReceived ( MessageId messageId , boolean confirmation )");
		this.messageId.setText(Integer.toString(messageId.view) + "." + Integer.toString(messageId.id));
		messageConfirmed.setSelected(confirmation);
	}

	public void print(final String message) {
		Trace.code("TraceGraphical.java -> void print ( final String message )");
		if (printMessages)
			printAlways(message);
	}

	public void printMembers(String message, int members[]) {
		Trace.code("TraceGraphical.java -> void printMembers ( String message , int members [ ] )");
		StringBuffer s = new StringBuffer(message + " Members : [ ");
		for (int i = 0; i < members.length; i++) {
			if (i > 0)
				s.append(" - ");
			s.append(members[i]);
		}
		s.append(" ]");
		printAlways(s.toString());
	}

	void printAlways(final String message) {
		Trace.code("TraceGraphical.java -> void printAlways ( final String message )");
		new Thread() {
			public void run() {
				JOptionPane.showMessageDialog(TraceGraphical.this, message, "Trace", JOptionPane.INFORMATION_MESSAGE);
			}
		}
		.start();
	}

}
