/**
  * File: SendViewChannelRequester.java
  * Content: Defines a channel requester to send a view with wrong behaviour
  * Author: LuisM Pena
  * Date: 12th August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.Token;

/**
  * Channel Requester used to send views with wrong behaviour
  **/
class WrongSendViewChannelRequester extends SendViewChannelRequester {

	/**
	  * @param sentToAllButOne if this parameter is set to true, the messages are
		  *      	sent to every member but the first one; otherwise, it's only sent to the
		  *      	first one. (The owner will also receive it)
	  **/
	public WrongSendViewChannelRequester(int owner, boolean sendToAllButOne) {
		super(owner);
		Trace.code("WrongSendViewChannelRequester.java -> WrongSendViewChannelRequester ( int owner , boolean sendToAllButOne )");
		this.sendToAllButOne = sendToAllButOne;
		invocation = 0;
	}

	public void setInfoToSend(InternalView view, Token token) {
		Trace.code("WrongSendViewChannelRequester.java -> void setInfoToSend ( InternalView view , Token token )");
		super.setInfoToSend(view, token);
		//		invocation = 0;	Otherwise, the system just fall down (strategic attack!)
		Trace.code("WrongSendViewChannelRequester.java -> void setInfoToSend ( InternalView view , Token token )");
	}

	public boolean invokeMember(CommunicationChannel channel) {
		Trace.code("WrongSendViewChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
		assert channel != null;
		boolean invokeSuper = false;
		;
		if (channel.getLinkId() == thisMember)
			invokeSuper = true;
		else if (invocation++ == 0)
			invokeSuper = !sendToAllButOne;
		else
			invokeSuper = sendToAllButOne;
		return invokeSuper ? super.invokeMember(channel) : false;
	}

	int invocation;
	boolean sendToAllButOne;
}
