/**
  * File: LoggerServer.java
  * @author LuisM Pena
  * Version: 0.97.00
  **/

package sensei.util.logging;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.text.DateFormat;
import java.util.Date;

/**
  * Class implementing the logger server functionality
  **/
public class LoggerServer extends Thread
{

  /*******************************************************************************/
  /**************************** CONSTRUCTOR **************************************/
  /*******************************************************************************/

  /**
    * A LoggerServer needs a socket where it is published
    **/
  public LoggerServer(int socket) throws IOException
  {
    serverSocket=new ServerSocket(socket);
    dateFormat = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM);
    message("Logger server activated on port " + socket);
    initialTime = System.currentTimeMillis();
    new HandleInput().start(); //read keys, doing nothing
    start();
  }

  /*******************************************************************************/
  /**************************** MESSAGE ******************************************/
  /*******************************************************************************/

  void message(String msg)
  {
    System.out.println(dateFormat.format(new Date()) + " : " + msg);
  }

  /*******************************************************************************/
  /**************************** RUN **********************************************/
  /*******************************************************************************/

  /**
    * Thread method
    **/
  public void run()
  {
    while (true)
      try
      {
        Socket session = serverSocket.accept();
        new LoggingSession(this, session, initialTime);
      }
      catch(Exception ex)
      {
        message("Logger Server error: " + ex.getMessage());
      }
  }

  /*******************************************************************************/
  /******************** INNER CLASS **********************************************/
  /*******************************************************************************/

  class HandleInput extends Thread
  {
    public void run()
    {
      try
      {
        byte b[]=new byte[1];
        while (true)
        {
          System.in.read(b);
          switch(b[0])
          {
            case 'R':
            case 'r':
              System.out.println("Reset time? Press Y to reset time");
              break;
            case 'Y':
            case 'y':
              System.out.println("Logger time has been reset");
              initialTime = System.currentTimeMillis();
              break;
            default:
              break;
          }
        }
      }
      catch(Exception ex)
      {
      }
    }
  }

  /*******************************************************************************/
  /*******************************************************************************/
  /*******************************************************************************/

  ServerSocket serverSocket;
  DateFormat dateFormat;
  long initialTime;

  /*******************************************************************************/
  /*******************************************************************************/
  /*******************************************************************************/

  static public void main(String args[])
  {
    if (args.length!=1)
      System.out.println("The listening socket port must be specified as only argument");
    else
      try
      {
        new LoggerServer(Integer.valueOf(args[0]).intValue());
      }
      catch(NumberFormatException ex)
      {
        System.out.println(args[0] + " is not a valid number");
      }
      catch(IOException  ex)
      {
        System.out.println(ex.getMessage());
      }
  }

}
